
with(Groebner):
infolevel[GroebnerBasis]:=2:

#
SudokuGroebner:=proc(filledVals,blockDimRow,blockDimCol)
	description "Calculates Groebner basis of Sudoku whose",
		"constraints are specified by filledVals.";
	local eqs,tord;
	global blockdimrow,blockdimcol,numsize;
	
	blockdimrow:=blockDimRow;
	blockdimcol:=blockDimCol;
	numsize:=blockdimrow*blockdimcol;
	
	eqs:=AllConstraint(filledVals);
	tord:=VarsTord();
	Basis(eqs,tord,method='fgb');
end:

#
VarsTord:=proc()
	description "Creates torder";
	tdeg(seq(seq(x[i,j],j=1..numsize),i=1..numsize));
end:

Vars:=proc()
	description "All variables";
	[seq(seq(x[i,j],j=1..numsize),i=1..numsize)];
end:

AllConstraint:=proc(filledVals)
	description "Prepares all constraints";
	local clist;
	clist:=[seq(x[op(i[1..2])]-i[3],i=filledVals)];
	
	clist:=[op(clist),op(CellConstraint())];
	clist:=[op(clist),op(RowColBlockConstraint())];
	clist;
end:

#
EachCell:=proc(i,j)
	description "Each cell must have one value from 1..numsize";
	product(x[i,j]-k,k=1..numsize);
end:

#
CellConstraint:=proc()
	description "All cells must have one value from 1..numsize";
	[seq(seq(EachCell(i,j),i=1..numsize),j=1..numsize)];
end:

#
BlockConstraint:=proc(indicesListList)
	description "Each row, col and block does not have multiple values";
	local p,s;
	p:=mul(i,i=1..numsize);
	s:=add(i,i=1..numsize);
	[mul(x[op(cor)],cor=indicesListList)-p,add(x[op(cor)],cor=indicesListList)-s];
end:

#
RowColBlockConstraint:=proc()
	description "Constraints for row, col and blocks";
	local i,j,v,clist;
	clist:=[];
	for i from 1 to numsize do
		v:=[seq([i,j],j=1..numsize)];
		clist:=[op(clist),op(BlockConstraint(v))];
	end;
	
	for i from 1 to numsize do
		v:=[seq([j,i],j=1..numsize)];
		clist:=[op(clist),op(BlockConstraint(v))];
	end;
	
	for i from 1 to numsize/blockdimrow do
		for j from 1 to numsize/blockdimcol do
			v:=BlockIndices(i,j);
			clist:=[op(clist),op(BlockConstraint(v))];
		end;
	end;
	
	clist;
end:

BlockIndices:=proc(blockRow,blockCol)
	description "Indices of each rectangle block";
	[seq(seq([(blockRow-1)*blockdimrow+i,(blockCol-1)*blockdimcol+j],j=1..blockdimcol),i=1..blockdimrow)];
end:

